/* Stream server */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/wait.h>
#include <signal.h>
#include <time.h>

#define MYPORT 10000
#define BACKLOG 10 /* Queue */

void sigchld_handler(int s)
{
    while(waitpid(-1, NULL, WNOHANG) > 0);
}

int main(void)
{
    int sockfd, new_fd; /* listen on sock_fd, new connection on new_fd */
    struct sockaddr_in my_addr; /* my addr info */
    struct sockaddr_in their_addr; /* connector addr info */
    socklen_t sin_size;
    struct sigaction sa;
    int yes=1;
    char buf[128];
    int numbytes;

    if ((sockfd = socket(AF_INET, SOCK_STREAM, 0)) == -1) {
        perror("socket");
        exit(1);
    }

    if (setsockopt(sockfd,SOL_SOCKET,SO_REUSEADDR,&yes,sizeof(int)) == -1) {
        perror("setsockopt");
        exit(1);
    }

    my_addr.sin_family = AF_INET; /* host byte order */
    my_addr.sin_port = htons(MYPORT); /* short, network byte order */
    my_addr.sin_addr.s_addr = INADDR_ANY; /* automatically fill with my IP */
    memset(&(my_addr.sin_zero), '\0', 8); /* zero the rest of the struct */

    if (bind(sockfd, (struct sockaddr *)&my_addr, sizeof(struct sockaddr))
        == -1) {
        perror("bind");
        exit(1);
    }

    if (listen(sockfd, BACKLOG) == -1) {
        perror("listen");
        exit(1);
    }

    printf("== %i == server: listening on port %i\n", time(0), MYPORT);
    sa.sa_handler = sigchld_handler; /* reap all dead processes */
    sigemptyset(&sa.sa_mask);
    sa.sa_flags = SA_RESTART;

    if (sigaction(SIGCHLD, &sa, NULL) == -1) {
        perror("sigaction");
        exit(1);
    }

    while(1) { /* main accept() loop */

        sin_size = sizeof(struct sockaddr_in);

        if ((new_fd = accept(sockfd, (struct sockaddr *)&their_addr,
                             &sin_size)) == -1) {
            perror("accept");
            continue;
        }

        printf("== %i == server: got connection from %s\n", time(0), inet_ntoa(their_addr.sin_addr));

        numbytes = recv(new_fd, buf, 128, 0);
        buf[numbytes] = '\0';
        printf("== %i == server: received %i bytes => %s\n", time(0), numbytes, buf);

        if (!fork()) { /* child process */

            close(sockfd); /* child does not need the listener */

            printf("== %i == server: sending back %i bytes => %s\n", time(0), numbytes, buf);
            if (send(new_fd, buf, numbytes, 0) == -1)
                perror("send");
            close(new_fd);

            exit(0);
        }
        close(new_fd); /* parent does not need this */
    }
    return 0;
}
